package com.picoinstruments.android.simplecamera;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.Date;

import android.app.Activity;
import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.content.pm.ActivityInfo;
import android.content.res.Configuration;
import android.media.AudioManager;
import android.media.MediaPlayer;
import android.net.Uri;
import android.os.Bundle;
import android.os.Environment;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.os.Messenger;
import android.os.RemoteException;
import android.util.Log;
import android.view.SurfaceView;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.widget.Toast;

public class SimpleCameraActivity extends Activity {
    private static String TAG = "SimpleCameraActivity";

    private Context mContext;
    private SurfaceView mVideoVideo;
    private boolean mCameraServiceBound = false;
    private boolean mVideoStarted = false;
    private String mStoragePath;

    /* the class name of the service we will bind */
    private static final String CAMERA_SERVICE_NAME = "com.picoinstruments.android.icubiecamera.CameraService";

    /* bundle used to send objects to Camera Service which runs in a separate process */
    private final Bundle mBundle = new Bundle();
    /* Messenger used by clients to send messages to the camera service. */
    private Messenger mCameraService =  null;
    /* Messenger used by the camera service to send messages to client's IncomingHandler(). */
    final Messenger mMessenger = new Messenger(new IncomingHandler());

    private ServiceConnection mConnection = new ServiceConnection() {
        public void onServiceConnected(ComponentName className,
                IBinder service) {
            /* register with the camera service in order to receive its messages */
            mCameraService = new Messenger(service);
            try {
                Message msg = Message.obtain(null, CameraService.REQ_REGISTER_CLIENT);
                msg.replyTo = mMessenger;
                mCameraService.send(msg);
            } catch (RemoteException e) {
            }
            Log.i(TAG, "SimpleCameraActivity bound to CameraService");
        }

        public void onServiceDisconnected(ComponentName className) {
            mCameraService = null;
            Log.w(TAG, "CameraService disconnected unexpectedly");
        }
    };

    /* Handler of incoming messages from the camera service. */
    private class IncomingHandler extends Handler {
        @Override
        public void handleMessage(Message msg) {
            switch (msg.what) {
                case CameraService.IND_VIDEO_STARTED:
                    mVideoStarted = (msg.arg1 != 0);
                    Log.d(TAG, mVideoStarted ? "Video started" : "Video stopped");
                    break;

                case CameraService.RSP_TAKE_PHOTO:
                    switch(msg.arg1) {
                        case CameraService.ERROR_CODE_NONE:
                            /* give visual audible indication of a successful photo taking */
                            playCameraShutterSound();
                            /* retrieve the file name sent from the Camera Service */
                            Bundle bundle = msg.getData();
                            String filename = bundle.getString("FromCameraService");
                            File photo = new File(filename);
                            Toast.makeText(mContext, photo.getName(), Toast.LENGTH_SHORT).show();
                            break;
                        case CameraService.ERROR_CODE_FILE_NOT_FOUND_EXECEPTION:
                            Toast.makeText(mContext, "Failed to access external storage device, file not found.", Toast.LENGTH_SHORT).show();
                            break;
                        case CameraService.ERROR_CODE_IO_EXECEPTION:
                            Toast.makeText(mContext, "Failed to access external storage device, i/o error.", Toast.LENGTH_SHORT).show();
                            break;
                    }
                    break;
            }
        }
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        mContext = this;

        /* Hide the window title, set full screen with Landscape orientation. */
        requestWindowFeature(Window.FEATURE_NO_TITLE);
        getWindow().addFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN);
        setContentView(R.layout.activity_simple_camera);
        setRequestedOrientation(ActivityInfo.SCREEN_ORIENTATION_LANDSCAPE);
        getWindow().addFlags(WindowManager.LayoutParams.FLAG_KEEP_SCREEN_ON);

        /* place we save captured picture */
        File storageDir = new File(Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_PICTURES) + "/iCubieCamera");
        storageDir.mkdirs();
        mStoragePath = storageDir.toString();

        /* take a picture action */
        mVideoVideo = (SurfaceView)findViewById(R.id.cp);
        mVideoVideo.setOnClickListener(new View.OnClickListener() {
            public void onClick(View v) {
                if(mCameraService != null)
                {
                    if(mVideoStarted)
                    {
                        /* Create an image file name */
                        String timeStamp = new SimpleDateFormat("yyyy-MM-dd HH.mm.ss").format(new Date());
                        String imageFileName = "IMG_" + timeStamp + ".jpg";
                        /* open file for write */
                        File image = new File(mStoragePath, imageFileName);
                        /* request the camera service to capture and save a JPEG picture */
                        Message msg = Message.obtain(null, CameraService.REQ_TAKE_PHOTO, 0, 0);
                        mBundle.putString("ToCameraService", image.toString());
                        msg.setData(mBundle);
                        msg.replyTo = mMessenger;    // it is needed in order for the camera to identify which client is sending this message
                        try {
                            mCameraService.send(msg);
                        } catch (RemoteException e) {
                        }
                    } else Toast.makeText(mContext, "Camera video not started.", Toast.LENGTH_SHORT).show();
                }
            }
        });
    }

    @Override
    protected void onStart() {
        super.onStart();
        if(getResources().getConfiguration().orientation == Configuration.ORIENTATION_LANDSCAPE) {
            /* bind the camera service */
            bindService(new Intent(CAMERA_SERVICE_NAME), mConnection, Context.BIND_AUTO_CREATE);
            mCameraServiceBound = true;
        }
    }

    @Override
    protected void onStop() {
        super.onStop();
        // unregister the camera service
        if (mCameraService != null) {
            try {
                Message msg = Message.obtain(null, CameraService.REQ_UNREGISTER_CLIENT);
                msg.replyTo = mMessenger;
                mCameraService.send(msg);
            } catch (RemoteException e) {
            }
            mCameraService = null;
        }

        // Unbind from the service
        if(mCameraServiceBound) {
            unbindService(mConnection);
            mCameraServiceBound = false;
            Log.i(TAG, "Unbinding SimpleCameraActivity from camera service");
        }
    }

    private void playCameraShutterSound()
    {
        AudioManager mAudioManager = (AudioManager)getSystemService(Context.AUDIO_SERVICE);
        if(0 != mAudioManager.getStreamVolume(AudioManager.STREAM_NOTIFICATION))
        {
            /* make audible sound */
            MediaPlayer mMediaPlayer = MediaPlayer.create(mContext, Uri.parse("file:///system/media/audio/ui/camera_click.ogg"));
            if(mMediaPlayer != null) mMediaPlayer.start();
        }
    }
}
