package com.picoinstruments.android.simplecamera;

import android.content.ComponentName;
import android.content.Context;
import android.content.Intent;
import android.content.ServiceConnection;
import android.graphics.Bitmap;
import android.graphics.Canvas;
import android.graphics.Rect;
import android.os.Handler;
import android.os.IBinder;
import android.os.Message;
import android.os.Messenger;
import android.os.RemoteException;
import android.util.AttributeSet;
import android.util.Log;
import android.view.SurfaceHolder;
import android.view.SurfaceView;

public class SimpleCameraPreview extends SurfaceView implements
        SurfaceHolder.Callback {
    private static String TAG = "SimpleCameraPreview";

    private static final int VIDEO_WIDTH = 640;
    private static final int VIDEO_HEIGHT = 480;

    private SurfaceHolder mHolder;
    private Rect mRectVideoSize;
    private Rect mRectFullScreen;

    /* the class name of the service we will bind */
    private static final String CAMERA_SERVICE_NAME = "com.picoinstruments.android.icubiecamera.CameraService";
    
    /* Messenger used by clients to send messages to the camera service. */
    private Messenger mCameraService =  null;
    /* Messenger used by the camera service to send messages to client's IncomingHandler(). */
    final Messenger mMessenger = new Messenger(new IncomingHandler());

    /* Handler of incoming messages from the camera service. */
    private class IncomingHandler extends Handler {
        @Override
        public void handleMessage(Message msg) {
            switch (msg.what) {
                case CameraService.IND_VIDEO_STARTED:
                    if(msg.arg1 != 0)
                    {
                        msg = Message.obtain(null, CameraService.REQ_GET_FRAME, 1, 0);
                        msg.replyTo = mMessenger;    // it is needed in order for the camera to identify which client is sending this message
                        try {
                            mCameraService.send(msg);
                        } catch (RemoteException e) {
                        }
                    }
                break;

                case CameraService.IND_GET_FRAME:
                    Bitmap bitmap = (Bitmap)msg.obj;
                    Canvas canvas = mHolder.lockCanvas();
                    if(canvas != null) {
                        if(bitmap != null) {
                            mRectVideoSize.right = bitmap.getWidth();
                            mRectVideoSize.bottom = bitmap.getHeight();
                            canvas.drawBitmap(bitmap, mRectVideoSize, mRectFullScreen, null);
                        }
                        mHolder.unlockCanvasAndPost(canvas);
                    }
                break;

                default:
                    super.handleMessage(msg);
            }
        }
    }

    public SimpleCameraPreview(Context context) {
        super(context);
        surfaceViewSetup();
    }

    public SimpleCameraPreview(Context context, AttributeSet attrs) {
        super(context, attrs);
        surfaceViewSetup();
    }

    private void surfaceViewSetup() {
        mHolder = getHolder();
        mHolder.addCallback(this);
        mRectVideoSize = new Rect(0, 0, 0, 0);
        mRectFullScreen = new Rect(0, 0, 0, 0);
    }

    protected Rect getViewingWindow() {
        return mRectFullScreen;
    }

    @Override
    public void surfaceCreated(SurfaceHolder holder) {
        Log.d(TAG, "surfaceCreated()");
        /* bind the camera service */
        getContext().bindService(new Intent(CAMERA_SERVICE_NAME),
                mConnection, Context.BIND_AUTO_CREATE);
    }

    @Override
    public void surfaceDestroyed(SurfaceHolder holder) {
        Log.d(TAG, "surfaceDestroyed()");

        /* unregister the camera service */
        if (mCameraService != null) {
            try {
                Message msg = Message.obtain(null, CameraService.REQ_UNREGISTER_CLIENT);
                msg.replyTo = mMessenger;
                mCameraService.send(msg);
            } catch (RemoteException e) {
                /* There is nothing special we need to do if the service has crashed. */
            }
        }
        /* unbind the camera service */
        getContext().unbindService(mConnection);
        mCameraService = null;
        Log.i(TAG, "Unbinding CameraPreview from camera service");
    }

    @Override
    public void surfaceChanged(SurfaceHolder holder, int format, int winWidth,
            int winHeight) {
        Log.d(TAG, "surfaceChanged():" + winWidth + "x" + winHeight);

        mRectFullScreen.right = (VIDEO_WIDTH * winHeight) / VIDEO_HEIGHT;
        mRectFullScreen.bottom = winHeight;
        mRectFullScreen.left = (winWidth - mRectFullScreen.right) / 2;
        mRectFullScreen.right += mRectFullScreen.left;
    }

    private ServiceConnection mConnection = new ServiceConnection() {
        public void onServiceConnected(ComponentName className,
                IBinder service) {
            /* register with the camera service in order to receive its messages */
            mCameraService = new Messenger(service);
            try {
                Message msg = Message.obtain(null, CameraService.REQ_REGISTER_CLIENT);
                msg.replyTo = mMessenger;
                mCameraService.send(msg);
            } catch (RemoteException e) {
            }
            Log.i(TAG, "CameraPreview bound to CameraService");
        }

        public void onServiceDisconnected(ComponentName className) {
            mCameraService = null;
            Log.w(TAG, "CameraService disconnected unexpectedly");
        }
    };
}
